import unittest
import json
import http.server
import os
import threading
import requests
import base64
import hashlib

from ttproto.ts_coap import webserver
from ttproto.ts_coap.webserver import *


class WebserverTestCase(unittest.TestCase):
    """
        Test case for the Webserver
        Mostly of the API calls
    """

    # #################### Tests parameters #########################

    # Webserver binding
    SERVER_ADDRESS = '0.0.0.0'
    SERVER_PORT = 8008
    TAT_API_URL = 'http://127.0.0.1:' + str(SERVER_PORT)
    FILES_DIR = 'tests/test_files'

    # Accepted values for some fields
    VERDICT_VALUES = [None, "inconclusive", "pass", "fail", "error"]

    # Some dummy values
    EXISTING_TEST_CASE = 'TD_COAP_CORE_01'
    UNKNOWN_TEST_CASE = 'UNKNOWN_TEST_CASE'
    WRONG_TOKEN = 'ayVMgJQiQICOCqBKE7pV7qVzU6k'
    WRONG_FRAME_ID = 500
    JOKER_PROTOCOL_NAME = 'None'

    # Put some test parameters passed from one function to another
    CORRECT_TOKEN = None
    CORRECT_FRAME_ID = None

    # Structure definition of packets
    STRUCT_RESPONSE_OK = {
        '_type': str,
        'ok': bool,
        'content': list
    }
    STRUCT_RESPONSE_KO = {
        '_type': str,
        'ok': bool,
        'error': str
    }
    STRUCT_TC_BASIC = {
        '_type': str,
        'id': str,
        'objective': str
    }
    STRUCT_TC_IMPLEMENTATION = {
        '_type': str,
        'implementation': str
    }
    STRUCT_IMPLEMENTED_PROTOCOL = {
        '_type': str,
        'name': str,
        'description': str
    }
    STRUCT_TOKEN = {
        '_type': str,
        'value': str
    }
    STRUCT_FRAME = {
        '_type': str,
        'id': int,
        'timestamp': float,
        'error': (type(None), str),
        'protocol_stack': list
    }
    STRUCT_VERDICT = {
        '_type': str,
        'verdict': str,
        'description': str,
        'review_frames': list
    }

    # #################### Init and deinit functions #########################
    # def setUp(self):
    #     """
    #         Initialize the server on which we'll run the tests
    #         It creates a thread on which we run the server
    #     """
    #     # Put the needed log file for the webserver (it's required)
    #     webserver.log_file = open(
    #         os.path.join(LOGDIR, 'unit-tests-webserver.log'),
    #         'a'
    #     )

    #     # Create the server instance
    #     self.server = http.server.HTTPServer(
    #         (
    #             self.SERVER_ADDRESS,
    #             self.SERVER_PORT
    #         ),
    #         RequestHandler)

    #     # Make a thread to handle server requests, the main one will do tests
    #     thread = threading.Thread(target=self.server.serve_forever)
    #     thread.start()

    # def tearDown(self):
    #     """
    #         Close the server
    #     """
    #     self.server.shutdown()
    #     self.server.server_close()
    #     webserver.log_file.close()

    @classmethod
    def setUpClass(cls):
        """
            Initialize the server on which we'll run the tests
            It creates a thread on which we run the server
        """

        # Put the needed log file for the webserver (it's required)
        webserver.log_file = open(
            os.path.join(LOGDIR, 'unit-tests-webserver.log'),
            'a'
        )

        # Create the server instance
        cls.server = http.server.HTTPServer(
            (
                cls.SERVER_ADDRESS,
                cls.SERVER_PORT
            ),
            RequestHandler)

        # Make a thread to handle server requests, the main one will do tests
        thread = threading.Thread(target=cls.server.serve_forever)
        thread.start()

    @classmethod
    def tearDownClass(cls):
        """
            Close the server
        """
        cls.server.shutdown()
        cls.server.server_close()
        webserver.log_file.close()

    # #################### Utilities functions #########################

    def check_correct_structure(self, el, structure):
        # Check that it's a non empty dict
        self.assertIsInstance(el, dict)
        self.assertGreater(len(el), 0)

        # Check its fields
        self.assertEqual(el.keys(), structure.keys())

        # Check the type of all its fields
        for field in structure:
            self.assertIsInstance(el[field], structure[field])

    def check_correct_response_header(self, response):
        # Check the object type
        self.assertIsInstance(response, requests.models.Response)

        # Check the response code
        self.assertEqual(response.status_code, 200)

        # Check the headers
        self.assertEqual(
            response.headers['content-type'],
            'application/json;charset=utf-8'
        )

    def check_request_not_found_header(self, response):
        # Check the object type
        self.assertIsInstance(response, requests.models.Response)

        # Check the response code
        self.assertEqual(response.status_code, 404)

    def check_correct_response(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_RESPONSE_OK)

        # Check its values
        self.assertEqual(el['_type'], 'response')
        self.assertTrue(el['ok'])
        self.assertGreater(len(el['content']), 0)

    def check_error_response(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_RESPONSE_KO)

        # Check its values
        self.assertEqual(el['_type'], 'response')
        self.assertFalse(el['ok'])

    def check_tc_basic(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_TC_BASIC)

        # Check its type
        self.assertEqual(el['_type'], 'tc_basic')

    def check_tc_implementation(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_TC_IMPLEMENTATION)

        # Check its type
        self.assertEqual(el['_type'], 'tc_implementation')

    def check_implemented_protocol(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_IMPLEMENTED_PROTOCOL)

        # Check its type
        self.assertEqual(el['_type'], 'implemented_protocol')

    def check_token(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_TOKEN)

        # Check its type
        self.assertEqual(el['_type'], 'token')

        # Check that the value is a correct hash
        # Add '=' only for checking
        decoded = base64.urlsafe_b64decode(el['value'] + '=')
        self.assertEqual(len(decoded), hashlib.sha1().digest_size)

    def check_protocol(self, el):
        # Check that it's a non empty dict
        self.assertIsInstance(el, dict)
        self.assertGreater(len(el), 0)

        # Check its minimum fields
        self.assertIn('_type', el)
        self.assertIn('_protocol', el)
        self.assertEqual(el['_type'], 'protocol')

        # Check that it has more values than that
        self.assertGreater(len(el), 2)

    def check_frame(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_FRAME)

        # Check its type
        self.assertEqual(el['_type'], 'frame')

        # Check that we have a protocol stack
        self.assertGreater(len(el['protocol_stack']), 0)
        for prot in el['protocol_stack']:
            self.check_protocol(prot)

    def check_is_int_real(self, el):
        # Check its type
        self.assertEqual(type(el), int)

        # Check its value
        self.assertGreater(el, 0)

    def check_verdict(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_VERDICT)

        # Check its type
        self.assertEqual(el['_type'], 'verdict')

        # Check that the verdict given is contained in the accepted values
        self.assertIn(el['verdict'], self.VERDICT_VALUES)

        # Check that we have some review frames
        self.assertGreaterEqual(len(el['review_frames']), 0)
        for frame in el['review_frames']:
            self.check_is_int_real(frame)

    # #################### Tests functions #########################

    # ##### get_test_cases(testcase_id=None)
    def test_get_test_cases_all_selected(self):
        test_cases = get_test_cases()
        self.assertIsInstance(test_cases, dict)
        self.assertGreater(len(test_cases), 0)
        self.assertIsInstance(test_cases, dict)

    def test_get_test_cases_all_selected_with_none_value(self):
        test_cases = get_test_cases(None)
        self.assertIsInstance(test_cases, dict)
        self.assertGreater(len(test_cases), 0)
        self.assertIsInstance(test_cases, dict)

    def test_get_test_cases_only_one_existing_test_case(self):
        test_case = get_test_cases(self.EXISTING_TEST_CASE)
        self.assertIsInstance(test_case, dict)
        self.assertEqual(len(test_case), 2)

        # Check that we received a correct element
        self.assertIn('tc_basic', test_case)
        self.check_tc_basic(test_case['tc_basic'])

        self.assertIn('tc_implementation', test_case)
        self.check_tc_implementation(test_case['tc_implementation'])

    def test_get_test_cases_only_one_unknown_test_case(self):
        test_case = get_test_cases(self.UNKNOWN_TEST_CASE)
        self.assertIsNone(test_case)

    # -------------------------------------------------------------------------------

    # ##### analyzer_getTestCases
    def test_analyzer_get_test_cases(self):

        # Execute the request
        resp = requests.get(self.TAT_API_URL + '/api/v1/analyzer_getTestCases')

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the content only contains tc_basic
        self.assertGreater(len(resp['content']), 0)
        for content in resp['content']:
            self.check_tc_basic(content)

    # -------------------------------------------------------------------------------

    # ##### analyzer_getTestcaseImplementation
    def test_analyzer_get_testcase_implementation(self):

        # Prepare GET parameters
        params = {'testcase_id': self.EXISTING_TEST_CASE}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getTestcaseImplementation',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the content only contains a tc_implementation and a tc_basic
        self.assertEqual(len(resp['content']), 2)
        self.check_tc_basic(resp['content'][0])
        self.check_tc_implementation(resp['content'][1])

    def test_analyzer_get_testcase_implementation_no_params(self):

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getTestcaseImplementation'
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects GET parameters, expected '?testcase_id={string}'"
        )

    def test_analyzer_get_testcase_implementation_wrong_params(self):

        # Prepare GET parameters
        params = {'wrong_param_name': self.EXISTING_TEST_CASE}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getTestcaseImplementation',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects GET parameters, expected '?testcase_id={string}'"
        )

    def test_analyzer_get_testcase_implementation_more_params(self):

        # Prepare GET parameters
        params = {
            'testcase_id': self.EXISTING_TEST_CASE,
            'another_param_not_wanted': 'this_param_is_unwanted'
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getTestcaseImplementation',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects GET parameters, expected '?testcase_id={string}'"
        )

    def test_analyzer_get_testcase_implementation_post_data_instead_of_get(self):

        # Prepare GET parameters
        params = {'testcase_id': self.EXISTING_TEST_CASE}

        # Execute the request with the params as POST and not GET params
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getTestcaseImplementation',
            data=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects GET parameters, expected '?testcase_id={string}'"
        )

    def test_analyzer_get_testcase_implementation_post_and_get_data(self):

        # Prepare GET parameters
        params = {'testcase_id': self.EXISTING_TEST_CASE}

        # Execute the request with the params as POST and not GET params
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getTestcaseImplementation',
            data=params,
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # It passes but in practice there is no POST values into a GET request

        # Check the content only contains a tc_implementation and a tc_basic
        self.assertEqual(len(resp['content']), 2)
        self.check_tc_basic(resp['content'][0])
        self.check_tc_implementation(resp['content'][1])

    def test_analyzer_get_testcase_implementation_unknown_test_case(self):

        # Prepare GET parameters
        params = {'testcase_id': self.UNKNOWN_TEST_CASE}

        # Execute the request with the params as POST and not GET params
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getTestcaseImplementation',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Test case %s not found' % self.UNKNOWN_TEST_CASE
        )

    # -------------------------------------------------------------------------------

    # ##### analyzer_getProtocols
    def test_analyzer_get_protocols(self):

        # Execute the request
        resp = requests.get(self.TAT_API_URL + '/api/v1/analyzer_getProtocols')

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the content only contains protocols
        self.assertGreater(len(resp['content']), 0)
        for content in resp['content']:
            self.check_implemented_protocol(content)

    # -------------------------------------------------------------------------------

    # ##### analyzer_getFrames
    def test_analyzer_get_frames(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token and a frame
        self.assertEqual(len(resp['content']), 2)
        self.check_token(resp['content'][0])
        self.assertEqual(resp['content'][0]['value'], CORRECT_TOKEN)
        self.check_frame(resp['content'][1])
        self.assertEqual(resp['content'][1]['id'], CORRECT_FRAME_ID)

    def test_analyzer_get_frames_no_params(self):

        # Execute the request
        resp = requests.get(self.TAT_API_URL + '/api/v1/analyzer_getFrames')

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_only_token(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {'token': CORRECT_TOKEN}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_only_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {'frame_id': CORRECT_FRAME_ID}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_only_protocol_selection(self):

        # Prepare GET parameters
        params = {'protocol_selection': self.JOKER_PROTOCOL_NAME}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_only_token_and_protocol_selection(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token and a frame
        self.assertGreaterEqual(len(resp['content']), 2)
        self.check_token(resp['content'][0])
        self.assertEqual(resp['content'][0]['value'], CORRECT_TOKEN)

        # Check each following frames
        for i in range(1, len(resp['content'])):
            self.check_frame(resp['content'][i])

    def test_analyzer_get_frames_token_and_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': CORRECT_FRAME_ID
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_protocol_selection_and_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_frame_id_not_integer(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': 'definitively_not_an_integer'
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_frame_id_negative(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': -42
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_more_params(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': CORRECT_FRAME_ID,
            'testcase_id': self.EXISTING_TEST_CASE
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_analyzer_get_frames_post_and_get_data(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': CORRECT_FRAME_ID
        }

        # Execute the request with the params as POST and not GET params
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            data=params,
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # It passes but in practice there is no POST values into a GET request

        # Check the response contains a token and a frame
        self.assertEqual(len(resp['content']), 2)
        self.check_token(resp['content'][0])
        self.check_frame(resp['content'][1])

    def test_analyzer_get_frames_wrong_token(self):

        # Check params that we normally should get from last dissect
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': self.WRONG_TOKEN,
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Session identified by token %s not found' % self.WRONG_TOKEN
        )

    def test_analyzer_get_frames_wrong_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': self.WRONG_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'No frame with id=%u found' % self.WRONG_FRAME_ID
        )

    def test_analyzer_get_frames_wrong_protocol_selection(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': 'WrongProtocolSelection'
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/analyzer_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Unknown %s protocol' % 'WrongProtocolSelection'
        )

    # TODO
    # After the protocol selection is implemented, check that we only receive
    # frames of this protocol and not others anymore

    # -------------------------------------------------------------------------------

    # ##### dissector_getFrames
    def test_dissector_get_frames(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token and a frame
        self.assertEqual(len(resp['content']), 2)
        self.check_token(resp['content'][0])
        self.assertEqual(resp['content'][0]['value'], CORRECT_TOKEN)
        self.check_frame(resp['content'][1])
        self.assertEqual(resp['content'][1]['id'], CORRECT_FRAME_ID)

    def test_dissector_get_frames_no_params(self):

        # Execute the request
        resp = requests.get(self.TAT_API_URL + '/api/v1/dissector_getFrames')

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_only_token(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {'token': CORRECT_TOKEN}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_only_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {'frame_id': CORRECT_FRAME_ID}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_only_protocol_selection(self):

        # Prepare GET parameters
        params = {'protocol_selection': self.JOKER_PROTOCOL_NAME}

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_only_token_and_protocol_selection(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token and a frame
        self.assertGreaterEqual(len(resp['content']), 2)
        self.check_token(resp['content'][0])
        self.assertEqual(resp['content'][0]['value'], CORRECT_TOKEN)

        # Check each following frames
        for i in range(1, len(resp['content'])):
            self.check_frame(resp['content'][i])

    def test_dissector_get_frames_token_and_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': CORRECT_FRAME_ID
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_protocol_selection_and_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_frame_id_not_integer(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': 'definitively_not_an_integer'
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_frame_id_negative(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': -42
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_more_params(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': CORRECT_FRAME_ID,
            'testcase_id': self.EXISTING_TEST_CASE
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
        )

    def test_dissector_get_frames_post_and_get_data(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'frame_id': CORRECT_FRAME_ID
        }

        # Execute the request with the params as POST and not GET params
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            data=params,
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # It passes but in practice there is no POST values into a GET request

        # Check the response contains a token and a frame
        self.assertEqual(len(resp['content']), 2)
        self.check_token(resp['content'][0])
        self.check_frame(resp['content'][1])

    def test_dissector_get_frames_wrong_token(self):

        # Check params that we normally should get from last dissect
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': self.WRONG_TOKEN,
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Session identified by token %s not found' % self.WRONG_TOKEN
        )

    def test_dissector_get_frames_wrong_frame_id(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': self.WRONG_FRAME_ID,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'No frame with id=%u found' % self.WRONG_FRAME_ID
        )

    def test_dissector_get_frames_wrong_protocol_selection(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        global CORRECT_FRAME_ID
        self.assertIsNotNone(CORRECT_TOKEN)
        self.assertIsNotNone(CORRECT_FRAME_ID)

        # Prepare GET parameters
        params = {
            'token': CORRECT_TOKEN,
            'frame_id': CORRECT_FRAME_ID,
            'protocol_selection': 'WrongProtocolSelection'
        }

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getFrames',
            params=params
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Unknown %s protocol' % 'WrongProtocolSelection'
        )

    # TODO
    # After the protocol selection is implemented, check that we only receive
    # frames of this protocol and not others anymore

    # -------------------------------------------------------------------------------

    # ##### dissector_getProtocols
    def test_dissector_get_protocols(self):

        # Execute the request
        resp = requests.get(
            self.TAT_API_URL + '/api/v1/dissector_getProtocols'
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the content only contains protocols
        self.assertGreater(len(resp['content']), 0)
        for content in resp['content']:
            self.check_implemented_protocol(content)

    # -------------------------------------------------------------------------------

    # ##### do_GET handler
    def test_do_get_handler_not_found(self):

        # Execute a request on an unknown path
        resp = requests.get(self.TAT_API_URL + '/api/v1/unknow_path')
        self.check_request_not_found_header(resp)

    # -------------------------------------------------------------------------------

    # ##### analyzer_testCaseAnalyze
    def test_analyzer_test_case_analyze_from_pcap(self):

        # Prepare POST parameters
        datas = {'testcase_id': self.EXISTING_TEST_CASE}

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas,
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token and a frame
        self.assertEqual(len(resp['content']), 3)
        self.check_token(resp['content'][0])
        self.check_tc_basic(resp['content'][1])
        self.check_verdict(resp['content'][2])

        # Check that the verdict is correct too
        self.assertEqual(resp['content'][2]['verdict'], 'pass')

    def test_analyzer_test_case_analyze_from_token(self):

        # Check params that we normally should get from last dissect
        global CORRECT_TOKEN
        self.assertIsNotNone(CORRECT_TOKEN)

        # Prepare POST parameters
        datas = {
            'testcase_id': self.EXISTING_TEST_CASE,
            'token': CORRECT_TOKEN
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token and a frame
        self.assertEqual(len(resp['content']), 3)
        self.check_token(resp['content'][0])
        self.assertEqual(resp['content'][0]['value'], CORRECT_TOKEN)
        self.check_tc_basic(resp['content'][1])
        self.check_verdict(resp['content'][2])

        # Check that the verdict is correct too
        self.assertEqual(resp['content'][2]['verdict'], 'pass')

    def test_analyzer_test_case_analyze_no_post_datas(self):

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data={}
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Non empty POST datas and format of 'multipart/form-data' expected"
        )

    def test_analyzer_test_case_analyze_only_testcase_id(self):

        # Prepare POST parameters
        datas = {'testcase_id': self.EXISTING_TEST_CASE}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
        )

    def test_analyzer_test_case_analyze_only_pcap_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data={},
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
        )

    def test_analyzer_test_case_analyze_only_token(self):

        # Prepare POST parameters
        datas = {'token': self.WRONG_TOKEN}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
        )

    def test_analyzer_test_case_analyze_both_token_and_pcap_file(self):

        # Prepare POST parameters
        datas = {'token': self.WRONG_TOKEN}

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas,
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
        )

    def test_analyzer_test_case_analyze_all_the_three(self):

        # Prepare POST parameters
        datas = {
            'token': self.WRONG_TOKEN,
            'testcase_id': self.EXISTING_TEST_CASE
        }

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas,
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
        )

    def test_analyzer_test_case_analyze_with_get_parameters(self):

        # Prepare POST parameters
        datas = {
            'token': self.WRONG_TOKEN,
            'testcase_id': self.EXISTING_TEST_CASE
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas,
            params=datas
        )

        # Check headers
        self.check_request_not_found_header(resp)

    def test_analyzer_test_case_analyze_pcap_file_not_a_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )

        # Prepare POST parameters
        datas = {
            'token': self.WRONG_TOKEN,
            'testcase_id': self.EXISTING_TEST_CASE,
            'pcap_file': pcap_path
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
        )

    def test_analyzer_test_case_analyze_pcap_file_in_file_but_not_a_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )

        # Prepare POST parameters
        datas = {
            'token': self.WRONG_TOKEN,
            'testcase_id': self.EXISTING_TEST_CASE
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas,
            files={'pcap_file': pcap_path}
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
        )

    def test_analyzer_test_case_analyze_unknown_testcase_id(self):

        # Prepare POST parameters
        datas = {
            'testcase_id': self.UNKNOWN_TEST_CASE
        }

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas,
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Test case %s not found' % self.UNKNOWN_TEST_CASE
        )

    def test_analyzer_test_case_analyze_pcap_then_token(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )

        # Prepare POST parameters
        datas = {
            'testcase_id': self.EXISTING_TEST_CASE
        }
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas,
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token and a frame
        self.assertEqual(len(resp['content']), 3)
        self.check_token(resp['content'][0])
        self.check_tc_basic(resp['content'][1])
        self.check_verdict(resp['content'][2])

        # Check that the verdict is correct too
        self.assertEqual(resp['content'][2]['verdict'], 'pass')

        # Then do the same request with the token
        datas = {
            'testcase_id': self.EXISTING_TEST_CASE,
            'token': resp['content'][0]['value']
        }

        # Execute the request
        resp_token = requests.post(
            self.TAT_API_URL + '/api/v1/analyzer_testCaseAnalyze',
            data=datas
        )

        # Check headers
        self.check_correct_response_header(resp_token)

        # Check data headers
        resp_token = resp_token.json()
        self.check_correct_response(resp_token)

        # Check the response contains a token and a frame
        self.assertEqual(len(resp_token['content']), 3)
        self.check_token(resp_token['content'][0])
        self.check_tc_basic(resp_token['content'][1])
        self.check_verdict(resp_token['content'][2])

        # Check that the verdict is correct too
        self.assertEqual(resp_token['content'][2]['verdict'], 'pass')

        # Check both of the responses
        self.assertEqual(resp, resp_token)

    # -------------------------------------------------------------------------------

    # ##### dissector_dissectFile

    # The a is a little hack for this function to be the first tested
    # in order to get a valid token for the following test suite
    def test_01_dissector_dissect_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data={'protocol_selection': self.JOKER_PROTOCOL_NAME},
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_correct_response(resp)

        # Check the response contains a token
        self.assertGreaterEqual(len(resp['content']), 2)
        self.check_token(resp['content'][0])

        # Get the token then
        global CORRECT_TOKEN
        CORRECT_TOKEN = resp['content'][0]['value']

        # And many frames
        for i in range(1, len(resp['content'])):
            self.check_frame(resp['content'][i])

        # Get the last frame_id
        global CORRECT_FRAME_ID
        CORRECT_FRAME_ID = resp['content'][i]['id']

    def test_dissector_dissect_file_no_post_datas(self):

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data={}
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Non empty POST datas and format of 'multipart/form-data' expected"
        )

    def test_dissector_dissect_file_only_protocol_selection(self):

        # Prepare POST parameters
        datas = {'protocol_selection': self.JOKER_PROTOCOL_NAME}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data=datas
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "POST format of 'multipart/form-data' expected, no file input 'pcap_file' found"
        )

    def test_dissector_dissect_file_only_pcap_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data={},
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Expected POST=(pcap_file={file}, protocol_selection={text})'
        )

    def test_dissector_dissect_file_with_get_parameters(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )

        # Prepare POST parameters
        datas = {
            'pcap_file': pcap_path,
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data=datas,
            params=datas
        )

        # Check headers
        self.check_request_not_found_header(resp)

    def test_dissector_dissect_file_pcap_file_not_a_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )

        # Prepare POST parameters
        datas = {
            'protocol_selection': self.JOKER_PROTOCOL_NAME,
            'pcap_file': pcap_path
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data=datas
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "POST format of 'multipart/form-data' expected, no file input 'pcap_file' found"
        )

    def test_dissector_dissect_file_pcap_file_in_file_but_not_a_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )

        # Prepare POST parameters
        datas = {
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data=datas,
            files={'pcap_file': pcap_path}
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Expected 'pcap_file' to be a non empty pcap file"
        )

    def test_dissector_dissect_file_pcap_file_in_file_but_not_a_pcap_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s" % (
            self.FILES_DIR,
            'WrongFilesForTests',
            'not_a_pcap_file.dia'
        )

        # Prepare POST parameters
        datas = {
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data=datas,
            files={'pcap_file': pcap_path}
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Expected 'pcap_file' to be a non empty pcap file"
        )

    def test_dissector_dissect_file_empty_pcap_file(self):

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            'WrongFilesForTests',
            'empty_pcap'
        )

        # Prepare POST parameters
        datas = {
            'protocol_selection': self.JOKER_PROTOCOL_NAME
        }

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data=datas,
            files={'pcap_file': pcap_path}
        )

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            "Expected 'pcap_file' to be a non empty pcap file"
        )

    def test_dissector_dissect_file_unknown_protocol_selection(self):

        # Prepare POST parameters
        datas = {
            'protocol_selection': 'unknown_protocol_selection'
        }

        # Get the path of the pcap file
        pcap_path = "%s/%s/%s.pcap" % (
            self.FILES_DIR,
            self.EXISTING_TEST_CASE,
            'pass'
        )
        files = {'pcap_file': open(pcap_path, 'rb')}

        # Execute the request
        resp = requests.post(
            self.TAT_API_URL + '/api/v1/dissector_dissectFile',
            data=datas,
            files=files
        )

        # Close the file
        files['pcap_file'].close()

        # Check headers
        self.check_correct_response_header(resp)

        # Check data headers
        resp = resp.json()
        self.check_error_response(resp)

        # Check the content is an error message
        self.assertEqual(
            resp['error'],
            'Unknown protocol unknown_protocol_selection'
        )

    # -------------------------------------------------------------------------------

    # ##### do_POST handler
    def test_do_post_handler_not_found(self):

        # Execute a request on an unknown path
        resp = requests.post(self.TAT_API_URL + '/api/v1/unknow_path')
        self.check_request_not_found_header(resp)


# #################### Main run the tests #########################
if __name__ == '__main__':
    unittest.main()



from ..common import *
from ...core.templates import Range


class TD_COAP_BLOCK_03(CoAPTestCase):
    """
---
TD_COAP_BLOCK_03:
    obj: Handle PUT blockwise transfer for large resource
    cfg: CoAP_CFG_BASIC
    ref: '[BLOCK] 2.2, 2.3, 2.5'

    pre:
        - Client supports Block1 transfers
        - Server supports Block1 transfers
        - Server offers a large updatable resource /large-update

    seq:
    -   s: "Client is requested to update resource /large-update on
    Server"

    -   c:
        - 'Client sends a PUT request containing Block1 option
        indicating:'
        -   - NUM = 0
            - M = 1
            - SZX = the desired block size
            - Payload size is 2SZX+4 bytes

    -   c:
        - 'Server sends 2.04 (Changed) or 2.31 (Continue) response with a Block1 option
        indicating:'
        -   - NUM = 0
            - "M = 0 (stateless for 2.04) or 1 (atomic,for 2.31)"
            - SZX is less or equal to the SZX at step 2

    -   c:
        - 'Client sends further requests containing Block1 option
        indicating:'
        -   - NUM = "i where \\“i\\” is the block number of the current
                    block. If the server decreased the SZX parameter in
                    step 3, then the client should adapt the block size
                    accordingly and may resume the transfer from block
                    id 2size_in_step_2-size_in_step_3 instead of block 1)"
            - M = 1
            - SZX is the SZX at step 3
            - Payload size is 2SZX+4 bytes

    -   c:
        - 'Server sends 2.04 (Changed) or 2.31 (Continue) response containing Block1
        option indicating:'
        -   - NUM = i where “i” is the block number used at step 4
            - M = 0 (stateless, for 2.04) or 1 (atomic, for 2.31)
            - SZX is the SZX at step 3

    -   c:
        - 'Client send PUT request containing the last block and
        indicating:'
        -   - NUM = n where “n” is the last block number
            - M = 0
            - SZX is the SZX at step 3
            - Payload size is lesser or equal to 2SZX+4

    -   c:
        - 'Server sends 2.04 (Changed) response with a Block1 option
        indicating:'
        -   - NUM = n where “n” is the block number used at step 6
            - M = 0
            - SZX is the SZX at step 3

    -   v: "Server indicates presence of the complete updated resource
    /large-update"

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='put', opt=Opt(CoAPOptionUriPath("large-update"))),
        ]

    def run (self):
        # Step 2 - initial request from the client
        self.match("client", CoAP (code = "put",
                         opt = self.uri ("/large-update", CoAPOptionBlock1 (num=0, m=1))))
        client_szx  = self.coap["opt"][CoAPOptionBlock1]["szx"]
        client_size = 2**(client_szx+4)
        self.match("client", CoAP (pl = Length (bytes, client_size)), "fail")

        self.next_skip_ack()

        # Step 3 - response from the server possibly w/ block size negociation
        if not self.match("server", CoAP (code = Any (CoAPCode (2.04), 2.31),
                         opt = Opt (CoAPOptionBlock1 (num=0, szx=Range(int, 0, client_szx)))
                )):
            return
        server_szx  = self.coap["opt"][CoAPOptionBlock1]["szx"]
        server_size = 2**(server_szx+4)

        self.next_skip_ack()

        # next block should have num 1
        counter = itertools.count(1)

        if server_szx != client_szx:
            # client & server do not agree on the block size

            # server requested a smaller block size, num may be bigger
            if (self.match("client", CoAP (opt = Opt (CoAPOptionBlock1()))), None):
                num = self.coap["opt"][CoAPOptionBlock1]
                if num > 1:
                    counter = itertools.count(num)

        for i in counter:

            # Step 4 - send next block from client
            self.match("client", CoAP (code = "put",
                             opt = Opt (CoAPOptionBlock1 (num=i, szx=server_szx))
                    ))


            last = not self.match("client", CoAP (    opt = Opt (CoAPOptionBlock1 (m=1)),
                                    pl = Length (bytes, server_size)
                    ), None)
            if last:
                # Step 6 - last block
                self.match("client", CoAP (    opt = Opt (CoAPOptionBlock1 (m=0)),
                                    pl = Length (bytes, (1, server_size))
                        ), "fail")

            self.next_skip_ack()

            # Step 5 / Step 7 - ack from the server
            self.match("server", CoAP (code = Any (CoAPCode (2.04), 2.31),
                             opt = Opt (CoAPOptionBlock1 (num=i, szx=server_szx))
                    ))

            if last:
                self.next_skip_ack (optional = True)
                break

            self.next_skip_ack()



import os

from tests.test_tat.test_tat import TestAnalysisInteropTestCase


class TestAnalysisInteropObserveTestCase(TestAnalysisInteropTestCase):

    def setUp(self):
        self.pcap_dir_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), '../test_dumps/coap_observe')
        self.pcap_path_list = os.listdir(self.pcap_dir_path)



#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import code, inspect


class DebugContext:
	__depth = 0

	def __init__ (self, *k):
		self.__txt = " ".join([str(v) for v in k])
	
	def __enter__ (self):
		print (" " * DebugContext.__depth, "[Enter %s]" % self.__txt)
		DebugContext.__depth += 1
	
	def __exit__ (self, *ei):
		DebugContext.__depth -= 1
		print (" " * DebugContext.__depth, "[Leave %s]" % self.__txt,
			"" if ei[0] is None else "raised '%s'" % ei[1]
		)
	

def run_console():
	"""run a an interactive console in the local context of the calling function"""

	c = code.InteractiveConsole(inspect.stack()[1][0].f_locals)
	c.runcode("import rlcompleter, readline\nreadline.parse_and_bind('tab: complete')\n")
	c.interact()
	



from ..common import *


class TD_LWM2M_1_INT_102(CoAPTestCase):
    """
---
testcase_id: TD_LWM2M_1.0_INT_102
uri : http://openmobilealliance.org/iot/lightweight-m2m-lwm2m
configuration: LWM2M_CFG_01
objective: Test that the Client updates the registration information on the server.
pre_conditions:
  - The Client supports the minimum Configuration C.1.
  - Client is registred with the Server
  - The Server will be prepared to change the client lifetime registration to 20 sec to set the conditions for a Client UPDATE  operation
  - On option, the client will be prepared to update its registration before the registration time expires
references: 'OMA-ETS-LightweightM2M-V1_0_1-20170926-A'
sequence:
  - step_id: 'TD_LWM2M_1.0_INT_102_step_01'
    type: stimuli
    node : lwm2m_client
    description:
      - 'Client is requested a WRITE request (COAP PUT) to Server to set lifetime resource of the Server Object Instance to 20 sec'
      - - Type = 0 (CON)
        - Code = 3 (PUT)

  - step_id: 'TD_LWM2M_1.0_INT_102_step_02'
    type: check
    description:
      - 'The request sent by the client contains'
      - - Type=0 and Code=3
        - Non-empty payload= A Media-Type payload where each Object is described as a Link according to content-format option. 
        - content-format=text/plain
        - Uri-Path option = 1/0/1

  - step_id: 'TD_LWM2M_1.0_INT_102_step_03'
    type: check
    description:
      - 'LwM2M server sends response containing'
      - - Code = 2.04 (Changed)
        - content-format=text/plain
        - Non-empty payload

  - step_id: 'TD_LWM2M_1.0_INT_102_step_04'
    type: verify
    node: lwm2m_client
    description:
      - 'LwM2M client indicates successful operation'

  - step_id: 'TD_LWM2M_1.0_INT_102_step_05'
    type: stimuli 
    node: lwm2m_client
    description:
      - 'LwM2M client sends a Re-registration message (CoAP POST) to LwM2M server'
      - - Type = 0 (CON)
        - Code = 2 (POST)

  - step_id: 'TD_LWM2M_1.0_INT_102_step_06'
    type: check
    description:
      - 'Sent POST request contains'
      - - Type=0 and Code=2
        - Uri-Path option = the location of the created registration
        - Uri-Query option = 'lt={20}'

  - step_id: 'TD_LWM2M_1.0_INT_102_step_07'
    type: check
    description:
      - 'LwM2M server sends response containing'
      - - Code = 2.04 (Changed)

  - step_id: 'TD_LWM2M_1.0_INT_102_step_08'
    type: verify
    node: lwm2m_client
    description:
      - 'LwM2M client indicates successful operation'
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='put'), CoAP(type='con', code='post')]

    def run(self):
        self.match('client', CoAP(type='con', code='put', opt=self.uri('/1/0/1')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('40'))), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionAccept('0'))), 'fail')
        self.next()

        self.match('server', CoAP(type='con', code=Any(65, 68), pl=Not(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')

        self.next()

        self.match('client', CoAP(type='con', code='post', opt=self.uri()), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionUriQuery('lt=20'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code=Any(65, 68)), 'fail')




#!/usr/bin/env python3
#
#   (c) 2012    Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.    You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors    have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


import subprocess


def get_git_version():
    v = "(undef)"
    try:
        v = " ".join (str (subprocess.Popen (
            ['git', 'describe', '--tags'],
            stdout = subprocess.PIPE,
            stderr = subprocess.PIPE,
            close_fds = True,
        ).stdout.read(), "utf8", "replace").splitlines())
        if not v:
            # fall back to commit id
            v = " ".join (str (subprocess.Popen (
                "git log --format=format:%h -1 --abbrev-commit".split(),
                stdout = subprocess.PIPE,
                close_fds = True,
            ).stdout.read(), "utf8", "replace").splitlines())
    except Exception as e:
        pass
    return v



from ..common import *


class TD_LWM2M_1_INT_205(CoAPTestCase):
    """
---
testcase_id: TD_LWM2M_1.0_INT_205
uri: http://openmobilealliance.org/iot/lightweight-m2m-lwm2m
configuration: LWM2M_CFG_01
objective: Setting writable Resources of Server Object (ID:1) Instance 0 using Plain Text data format (0)
pre_conditions: 
  - The Client supports the configuration C.3 (A superset of C.1 where server objects implements {Default Minimum PEriod, Default Maximum Period, Disable Timeout} additional resources)  
  - The Client is supporting Plain Text data format (0)
  - Client is registred with the server
references: 'OMA-ETS-LightweightM2M-V1_0_1-20170926-A'
notes: This test case is composed by 48 steps. Please refer to TD_LWM2M_PRO.yaml file for more details.
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='put'), CoAP(type='con', code='get')]

    def run(self):
        self.match('server', CoAP(type='con', code='put', opt=self.uri('/1/0/2')), 'fail')
        self.match('server', CoAP(pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('O'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=Any(65, 68), pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='put', opt=self.uri('/1/0/3')), 'fail')
        self.match('server', CoAP(pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('O'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=Any(65, 68), pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='put', opt=self.uri('/1/0/5')), 'fail')
        self.match('server', CoAP(pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('O'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=Any(65, 68), pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/1/0/2')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11542'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11542'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/1/0/3')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11542'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11542'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/1/0/5')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11542'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11542'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='put', opt=self.uri('/1/0/2')), 'fail')
        self.match('server', CoAP(pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('O'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=Any(65, 68), pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='put', opt=self.uri('/1/0/3')), 'fail')
        self.match('server', CoAP(pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('O'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=Any(65, 68), pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='put', opt=self.uri('/1/0/5')), 'fail')
        self.match('server', CoAP(pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('O'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=Any(65, 68), pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('0'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/1/0/2')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11542'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11542'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/1/0/3')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11542'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11542'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/1/0/5')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11542'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11542'))), 'fail')




 #!/usr/bin/env python3

from ttproto.ts_coap.proto_specific import CoAPTestcase
from ttproto.ts_coap.proto_templates import *

class TD_COAP_CORE_02 (CoAPTestcase):
    """Identifier:
TD_COAP_CORE_02
Objective:
Perform DELETE transaction (CON mode)
Configuration:
CoAP_CFG_BASIC
References:
[COAP] 5.8.4,1.2,2.1,2.2,3.1

Pre-test
conditions:
•	Server offers a /test resource that handles DELETE

Test Sequence:
Step
Type
Description

1
Stimulus
Client is requested to send a DELETE request with:
•	Type = 0(CON)
•	Code = 4(DELETE)

2
Check
The request sent by the client contains:
•	Type=0 and Code=4
•	Client-generated Message ID (➔ CMID)
•	Client-generated Token (➔ CTOK)
•	Uri-Path option "test"

3
Check
Server sends response containing:
•	Code = 66(2.02 Deleted)
•	Message ID = CMID, Token = CTOK
•	Content-format option if payload non-empty
•	Empty or non-empty Payload

4
Verify
Client displays the received information
"""
class TD_COAP_CORE_02 (CoAPTestcase):
    """Identifier:
TD_COAP_CORE_02
Objective:
Perform DELETE transaction (CON mode)
Configuration:
CoAP_CFG_BASIC
References:
[COAP] 5.8.4,1.2,2.1,2.2,3.1

Pre-test
conditions:
•	Server offers a /test resource that handles DELETE

Test Sequence:
Step
Type
Description

1
Stimulus
Client is requested to send a DELETE request with:
•	Type = 0(CON)
•	Code = 4(DELETE)

2
Check
The request sent by the client contains:
•	Type=0 and Code=4
•	Client-generated Message ID (➔ CMID)
•	Client-generated Token (➔ CTOK)
•	Uri-Path option "test"

3
Check
Server sends response containing:
•	Code = 66(2.02 Deleted)
•	Message ID = CMID, Token = CTOK
•	Content-format option if payload non-empty
•	Empty or non-empty Payload

4
Verify
Client displays the received information
"""
class TD_COAP_CORE_02 (CoAPTestcase):
    """Identifier:
TD_COAP_CORE_02
Objective:
Perform DELETE transaction (CON mode)
Configuration:
CoAP_CFG_BASIC
References:
[COAP] 5.8.4,1.2,2.1,2.2,3.1

Pre-test
conditions:
•	Server offers a /test resource that handles DELETE

Test Sequence:
Step
Type
Description

1
Stimulus
Client is requested to send a DELETE request with:
•	Type = 0(CON)
•	Code = 4(DELETE)

2
Check
The request sent by the client contains:
•	Type=0 and Code=4
•	Client-generated Message ID (➔ CMID)
•	Client-generated Token (➔ CTOK)
•	Uri-Path option "test"

3
Check
Server sends response containing:
•	Code = 66(2.02 Deleted)
•	Message ID = CMID, Token = CTOK
•	Content-format option if payload non-empty
•	Empty or non-empty Payload

4
Verify
Client displays the received information
"""
    def run (self):
        self.match_coap ("client", CoAP (type="con", code="delete",
                        opt=self.uri ("/test")))
        CMID = self.frame.coap["mid"]
        CTOK = self.frame.coap["tok"]

        self.next()

        self.match_coap ("server", CoAP (code = 2.02, mid = CMID,tok=CTOK,))
        if self.match_coap ("server", CoAP(pl = Not(b"")),None):
            self.match_coap ("server", CoAP (
                        opt = Opt (CoAPOptionContentFormat()),
                ), "fail")





from ..common import *


class TD_LWM2M_1_INT_101(CoAPTestCase):
    """
---
testcase_id: TD_LWM2M_1.0_INT_101
uri: http://openmobilealliance.org/iot/lightweight-m2m-lwm2m
configuration: LWM2M_CFG_01
objective: Test that the Client registers with the server. 
pre_conditions:
  - The Client supports the minimum Configuration C.1.
  - - LWM2M Server Object (ID = 1) Instance 0 with mandatory resources and Short Server ID = 1
  - - LWM2M Security Object (ID = 0) Instance 0 with mandatory resources and Bootstrap Server = False
  - - LWM2M Device Object (ID = 3) Instance 0 with mandatory resources. 
  - Device is turned on. 
  - The bootstrap procedure has been completed or the required bootstrap information is available to the Client. 
references: 'OMA-ETS-LightweightM2M-V1_0_1-20170926-A'
sequence:
  - step_id: 'TD_LWM2M_1.0_INT_101_step_01'
    type: stimuli
    node: lwm2m_client
    description:
      - 'LwM2M client sends a Registration message (CoAP POST) to LwM2M server'
      - - Type = 0 (CON)
        - Code = 2 (POST)
        - Content-format option=application/link-format
        - Non-empty Payload

  - step_id: 'TD_LWM2M_1.0_INT_101_step_02'
    type: check
    description:
      - 'Sent POST request contains'
      - - Type=0 and Code=2
        - Non-empty payload= A Media-Type payload where each Object is described as a Link according to content-format option.
        - content-format=application/link-format
        - Uri-Path option = /rd
        - Uri-Query option = 'ep={endPointClientName}'
        - Uri-Query option = 'lt={lifetime}'
        - Uri-Query option = 'sms={MSISDN}'
        - Uri-Query option = 'lwm2m={version}'
        - Uri-Query option = 'b={binding}'

  - step_id: 'TD_LWM2M_1.0_INT_101_step_03'
    type: check
    description:
      - 'LwM2M server sends response containing'
      - - Code = 2.01 (Created)
        - content-format=application/link-format
        - LocationPath option = the path to use for updating or deleting the registration
        - Non-empty payload

  - step_id: 'TD_LWM2M_1.0_INT_101_step_04'
    type: verify
    node: lwm2m_client
    description:
      - 'LwM2M client indicates successful operation'
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='post')]

    def run(self):
        self.match('client', CoAP(type='con', code='post', opt=self.uri('/rd')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('40'))), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionAccept('40'))), 'fail')
        opts = list (filter ((lambda o: isinstance (o, CoAPOptionUriQuery)), self.frame.coap["opt"]))
        if len (opts) < 1:
            self.setverdict ("inconclusive", "expect at least one UriQuery option")
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('40'))), 'fail')


        self.next()

        self.match('server', CoAP(code=2.01, pl=Not(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionContentFormat('40'))), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionLocationPath())), 'fail')



from ..common import *


class TD_LWM2M_1_INT_103(CoAPTestCase):
    """
---
testcase_id: TD_LWM2M_1.0_INT_103
uri : http://openmobilealliance.org/iot/lightweight-m2m-lwm2m
configuration: LWM2M_CFG_01
objective: Test that the Client is able to deregister at the server.
pre_conditions:
  - Client is registred with the server
  - The Client supports the minimum Configuration C.1.
references: 'OMA-ETS-LightweightM2M-V1_0_1-20170926-A'
sequence:
  - step_id: 'TD_LWM2M_1.0_INT_103_step_01'
    type: stimuli
    node : lwm2m_server
    description:
      - 'LwM2M Server is requested to send EXECUTE command (COAP POST) on the Disable Resource of the Server Object (ID:1) Instance'
      - - Type = 0 (CON)
        - Code = 2 (POST)

  - step_id: 'TD_LWM2M_1.0_INT_103_step_02'
    type: check
    description:
      - 'The request sent by the lwm2m server contains'
      - - Type=0 and Code=2
        - Uri-Path option = 1/0/4


  - step_id: 'TD_LWM2M_1.0_INT_103_step_03'
    type: check
    description:
        - 'Lwm2m client sends response containing'
        - - Code = 2.04(Changed)

  - step_id: 'TD_LWM2M_1.0_INT_103_step_04'
    type: verify
    node: lwm2m_server
    description:
        - 'The lwm2m server receives the Sucess Message'

  - step_id: 'TD_LWM2M_1.0_INT_103_step_05'
    type: stimuli
    node : lwm2m_client
    description:
      - 'Client is requested to send a de-regestration (COAP DELETE) to Server'
      - - Type = 0 (CON)
        - Code = 4 (DELETE)

  - step_id: 'TD_LWM2M_1.0_INT_103_step_06'
    type: check
    description:
      - 'The request sent by the client contains'
      - - Type=0 and Code=4
        - Uri-Path = the location of the created registration


  - step_id: 'TD_LWM2M_1.0_INT_103_step_07'
    type: check
    description:
        - 'Server sends response containing'
        - - Code = 2.02(Deleted)

  - step_id: 'TD_LWM2M_1.0_INT_103_step_08'
    type: verify
    node: lwm2m_client
    description:
        - 'Client receives Success Message from the server'
        - 'Client is removed from the servers registration database'
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='post'), CoAP(type='con', code='delete')]

    def run(self):
        self.match('server', CoAP(type='con', code='post', opt=self.uri('/1/0/4')), 'fail')

        self.next()

        self.match('client', CoAP(type='con', code=Any(65, 68)), 'fail')
    
        self.next()

        self.match('client', CoAP(type='con', code='delete', opt=self.uri()), 'fail')
  
        self.next()

        self.match('server', CoAP(type='con', code=2.02, pl=(b'')), 'fail')




#!/usr/bin/env python3

from ..common import *


class TD_COAP_CORE_23(CoAPTestCase):
    """
---
TD_COAP_CORE_23:
    cfg: CoAP_CFG_BASIC
    obj: Perform PUT transaction containing the If-None-Match option (CON
        mode)
    pre:
    - Server offers a /create1 resource, which does not exist and can be
        created by the client
    - Client & server support If-Non-Match
    ref: '[COAP] 5.8.1, 5.10.7, 5.10.10, 12.1.12'
    seq:
    -   n: single creation
    -   s: "Client is requested to send a confirmable PUT request to server\
            \u2019s resource so as to atomically create the resource."
    -   c:
        - 'The request sent by the client contains:'
        -   - Type = 0 (CON)
            - Code = 3 (PUT)
            - "Client-generated Message ID (\u2794 CMID)"
            - "Client-generated Token (\u2794 CTOK)"
            - Content-format option
            - Uri-Path option "create1"
            - Option Type=If-None-Match
            - An arbitrary payload
    -   c:
        - 'Server sends response containing:'
        -   - Code = 2.01 (Created)
            - Message ID = CMID, Token = CTOK
            - Content-format option if payload non-empty
            - Empty or non-empty Payload
    -   v: Client displays the response and the server created a new resource
    -   n: concurrent creations
    -   s: "Client is requested to send a confirmable PUT request to server\
            \u2019s resource so as to atomically create the resource."
    -   c:
        - 'The request sent by the client contains:'
        -   - Type = 0 (CON)
            - Code = 3 (PUT)
            - "Another client-generated Message ID \u2260 CMID (\u2794 CMID2)"
            - "Client-generated Token which may or may not be \u2260 CTOK\
                \ (\u2794 CTOK2)"
            - Content-format option
            - Uri-Path option "create1"
            - Option Type=If-None-Match
            - An arbitrary payload
    -   c:
        - 'Server sends response containing:'
        -   - Code = 4.12 (Precondition Failed)
            - Message ID = CMID2, Token = CTOK2
            - Optional Content-format option
            - Empty or non-empty Payload
    -   v: Client displays the response
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]

        .. note::
            Check the number/value of the uri query options or not?
        """
        return [
            CoAP(type='con', code='put', opt=Opt(CoAPOptionUriPath("create1"))),  # Step 1
            CoAP(type='con', code='put', opt=Opt(CoAPOptionUriPath("create1")))  # Step 5
        ]

    request_uri = "/create1"

    def run(self):
        # Part A
        # Step 2
        self.match("client", CoAP(type="con", code="put",
                                  opt=Opt(
                                      CoAPOptionContentFormat(),
                                      CoAPOptionUriPath(self.request_uri[1:]),
                                      CoAPOptionIfNoneMatch(),
                                  ),
                                  pl=Not(b"")))

        CMID = self.coap["mid"]
        CTOK = self.coap["tok"]

        self.next_skip_ack()
        # Step 3
        if not self.match("server", CoAP(type=Any(CoAPType("con"), "ack"),
                                         code=2.01, mid=CMID, tok=CTOK)):
            raise self.Stop()

        if self.match("server", CoAP(pl=Not(b"")), None):
            self.match("server", CoAP(opt=Opt(CoAPOptionContentFormat()), ), "fail")

        self.next_skip_ack(optional=True)

        # Part B
        # Step 6
        self.match("client", CoAP(type="con",
                                  code="put",
                                  opt=Opt(
                                      CoAPOptionContentFormat(),
                                      CoAPOptionUriPath(self.request_uri[1:]),
                                      CoAPOptionIfNoneMatch(),
                                  ),
                                  pl=Not(b"")))
        CMID2 = self.coap["mid"]
        CTOK2 = self.coap["tok"]
        if CMID2 != b"":
            if CMID2 == CMID:
                self.set_verdict("fail", "Message ID should be different")
        if CTOK2 != b"":
            if CTOK2 == CTOK:
                self.set_verdict("fail", "Token should be different")

        self.next_skip_ack()
        # Step 7
        if not self.match("server", CoAP(type=Any(CoAPType("con"), "ack"),
                                         code=4.12,
                                         mid=CMID2,
                                         tok=CTOK2), 'fail'):
            raise self.Stop()

        self.next_skip_ack(optional=True)
